;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x636cf6085a20b5e909f6002258e9ddfc . "./code.cg2js/cg-common-control.fasl")
(#x24dc102888565d95e1e7d5625b544b93 . "./code.cg2js/cg-clipboard-stack.fasl")
(#x4091e652b9b60eb8e93b8f63604c43b8 . "./code.cg2js/cg-yes-no-string.fasl")
(#xec58cd57742d8cb5e68567a3591674a2 . "./code.cg2js/cg-clipboard.fasl")
(#x345b715274a491379693d1d94389b20a . "./code.cg2js/cg-dde-utils.fasl")
(#xecfe237f1ffff8b369d584dcb9a057e8 . "./code.cg2js/cg-editable-text.fasl")
(#x14b0bcb6fb59e265cc27f7f77fd2abbe . "./code.cg2js/cg-player.fasl")
(#xbb3b9d383d2b0b76fc7fc6b2dc0825ab . "./code.cg2js/cg-text-or-combo.fasl")
(#x67b523e16c450aab37035b6e2c87dd91 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x0f258c1a7c2012b94fd250beb8b28cb7 . "./code.cg2js/cg-graphics-context.fasl")
(#xe3a12573ef877950a9fa30c18854015a . "./code.cg2js/cg-choose-printer.fasl")
(#x63db61cdefc86f0a1fa281b4f6e84b44 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x28c52dd77a0142c8b8aa8d30463bfff8 . "./code.cg2js/cg-edit-in-place.fasl")
(#x95eef4631eec3bf121e2a9b5086890aa . "./code.cg2js/cg-split-bar.fasl")
(#x720f9de871598971f6bcece18a0687a0 . "./code.cg2js/cg-timer.fasl")
(#xd336419c29f3cdcebfd26fa136f313f8 . "./code.cg2js/cg-progress-indicator.fasl")
(#x151eb182624db1c87de64a1040a46f21 . "./code.cg2js/cg-object-editor.fasl")
(#x21e87f1d55b8558e891fc5427a528ba7 . "./code.cg2js/cg-fill-texture.fasl")
(#x3f8eb0268310ed027464c0cf2b85426f . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#xbf91849de090450421b99815d2ba6222 . "./code.cg2js/cg-os-window.fasl")
(#xd82af4546a39b69091e5d6dec1da0a3e . "./code.cg2js/cg-color-dialog.fasl")
(#x557d5c94f63ab7067bc78eed52cb9be2 . "./code.cg2js/cg-combo-box.fasl")
(#xcaca6da246f7d1d91e4f40ae4e0c16cc . "./code.cg2js/cg-directory-dialog.fasl")
(#xfce6aa42827a6735500d22f9c6804643 . "./code.cg2js/cg-tooltip.fasl")
(#x61bc7bf4220b0fa2b664c976c10f08a0 . "./code.cg2js/cg-lettered-menu.fasl")
(#x90b09553608f8b62098c35cfc69f7fae . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x1d1a1dbc2f0a11157d68d64054c4aafb . "./code.cg2js/cg-pixmap.fasl")
(#xf101cdee813d758aa870faa985c99b2c . "./code.cg2js/cg.fasl")
(#x7b1351341240fbed021db07bd08534c7 . "./code.cg2js/cg-class-slot-grid.fasl")
(#xf1e4fa60b9ad3df022bac1d8aeb18b91 . "./code.cg2js/cg-template-string.fasl")
(#x2247f322f5afaf92358833539e4e2491 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#xcc1742fe339dcb94b5cc6c00aae8aadb . "./code.cg2js/cg-check-box.fasl")
(#x273da6a3ea3c6871ee08e4e3df70e107 . "./code.cg2js/cg-toggling-widget.fasl")
(#x0174fb08af3f58cb7b8558332a0ff70a . "./code.cg2js/cg-radio-button.fasl")
(#xf7b88dbacef990c61e5fcdb94e5f7e43 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x06a31a7d13a4f83aae63c74b37c5740f . "./code.cg2js/cg-scaling-stream.fasl")
(#xfc2fcb824c5f32289979faa8f16d0e59 . "./code.cg2js/cg-multi-picture-button.fasl")
(#x15e5a4b77d220b4d1c267c479476a616 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x79e8f8272a764149de493f1092c14007 . "./code.cg2js/cg-grid-widget.fasl")
(#x0cafd7f84f10b5956b59a9ee7bd3def3 . "./code.cg2js/cg-text-edit-pane.fasl")
(#x700ea900679253359e074be6088b5a38 . "./code.cg2js/cg-item-list.fasl")
(#xd8d2f5ab80287778561ab8b914b7d80b . "./code.cg2js/cg-paren-matching.fasl")
(#x5821e2552e9abc26f73bde5b4e2e66f6 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x94abb62d0fb586ed9ab3bf6ed381cb34 . "./code.cg2js/cg-html-dialog.fasl")
(#x57032e70457377b25e9fa45bfe2ff3f6 . "./code.cg2js/cg-pixmap-widget.fasl")
(#x450e39a324a12660a3d3e4c6fa7b91c4 . "./code.cg2js/cg-project-window.fasl")
(#x284765e060a09567975a2343d4357891 . "./code.cg2js/cg-toolbar.fasl")
(#x6aa3228b6c93593754348160e9f118a0 . "./code.cg2js/cg-caret.fasl")
(#x99828394cef79705141f124ef92dfe31 . "./code.cg2js/cg-scroll-bar.fasl")
(#xafd6fa1de538a8a474854d3f6582f591 . "./code.cg2js/cg-web-browser.fasl")
(#x95e6f369cccce827eb512e31f3d547df . "./code.cg2js/cg-dropping-outline.fasl")
(#xc8411175aa3fad2dc693a0ce7fdc08f2 . "./code.cg2js/cg-font-dialog.fasl")
(#x71d7c0a69abc166954a6d924775d84c0 . "./code.cg2js/cg-icon-pixmap.fasl")
(#xca8d0b907e73515e7c7cad2df67df163 . "./code.cg2js/cg-yes-no-list.fasl")
(#x8c97b9a5a08f60274eb3e2d86f770dd8 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#xd8a188e7de1dfb88b45b44d49ff2773d . "./code.cg2js/cg-lamp.fasl")
(#x609527c4750e36bb66664efeb47e05b5 . "./code.cg2js/cg-drawable.fasl")
(#x2a2c69ea7c15225918f6d255e51fb2a6 . "./code.cg2js/cg-choice-list.fasl")
(#xae43fd76b7edd139346b5a65cf0111f9 . "./code.cg2js/cg-curve.fasl")
(#x9055ec925bf8e70028c6f5627e316ded . "./code.cg2js/cg-base.fasl")
(#x6307cf2f27706c413634b98fe119e774 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#xc7fd1b2dab6bbb3f1eb39533c75daeac . "./code.cg2js/cg-printing.fasl")
(#xebd87ee8a6e6f9566e6d7b646bd57069 . "./code.cg2js/cg-chart-or-plot.fasl")
(#x63035d9282fc4f9af08b389c1f36b559 . "./code.cg2js/cg-hotspot.fasl")
(#x86e0facbd29dbb59257fc6b56d448f57 . "./code.cg2js/cg-object-editor-layout.fasl")
(#x5fabeca26a8540426aaa1dcdf0f5e70f . "./code.cg2js/cg-utility-dialog.fasl")
(#xc8f583bebc627d353bfc61826ce5412e . "./code.cg2js/cg-shortcut-menu.fasl")
(#x6889326722c78bc82bca9e06b5d3cb1b . "./code.cg2js/cg-comtab.fasl")
(#x269146c47698bc3123a2935b8b68e1d8 . "./code.cg2js/cg-os-widget.fasl")
(#x9d218cbaf33d1d890d410e57e8caaf99 . "./code.cg2js/cg-icon.fasl")
(#x0811da301fe0868ee28ac2e6969f3dff . "./code.cg2js/cg-dev.fasl")
(#x416c8d27f5399f8d2dab9a6aa29e2d78 . "./code.cg2js/cg-calendar.fasl")
(#x611d022313a459c6f81686c272f60ebf . "./code.cg2js/cg-group-box.fasl")
(#xe187ba8c3a7fd946ceaa0fb06060ff6b . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#xf86cc84ddae2bee1a491bae139ddf82a . "./code.cg2js/cg-bitmap-pane.fasl")
(#xf766c8772f5eb1864669c15782d176dd . "./code.cg2js/cg-drag-and-drop.fasl")
(#xb2c3ff102a5bc88eb62ccfdeb8bac9e9 . "./code.cg2js/cg-nodes.fasl")
(#x43b6f417f33e476cef93b4dc0952541c . "./code.cg2js/cg-intersect.fasl")
(#x53a03c5d5129e45258ad6549eceeab3b . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x15db5a9231ab7a6aa58de275bc4b149f . "./code.cg2js/cg-class-support.fasl")
(#xa789fbf343c544c17a0cff64a44587eb . "./code.cg2js/cg-dialog-item.fasl")
(#x436ef77cb453234100a20459c836ae77 . "./code.cg2js/cg-plot-widget.fasl")
(#x68911cfd8c42892b9a06118e726f739e . "./code.cg2js/cg-pixmap-printing.fasl")
(#xdc5abd7383ad27bbc6dabe87d08e01aa . "./code.cg2js/cg-up-down-control.fasl")
(#x4bf41f683e6c03f994f4b6058a2b6dcc . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x9f7b3e8fee646af99022703e43227d59 . "./code.cg2js/cg-status-bar.fasl")
(#xe9f9c7915da7f80dfc09abee9eae85cf . "./code.cg2js/cg-class-grid.fasl")
(#x872fa550fe8a2e0a0e5d6ac65a5b2376 . "./code.cg2js/cg-wrap-string.fasl")
(#x211622859f976664a0fd569ed55b0efe . "./code.cg2js/cg-layout.fasl")
(#x5d3f9985573d4e8d782619cfd02780f7 . "./code.cg2js/cg-lisp-widget.fasl")
(#xbd24861bf2d9c56b75f2d9738f2118bb . "./code.cg2js/cg-button.fasl")
(#x8c3070de3e4ceb651279decdd1476104 . "./code.cg2js/cg-string-dialog.fasl")
(#x46f0a303dfaeeac8b4bc92eaf1c36be0 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#xf873d96cb07b1a158b030259ecde5603 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#xd8de0b80705001494425e2ce6a86b42e . "./code.cg2js/cg-bitmap-stream.fasl")
(#x893cb1ce99604fe7cd8be6a309d542bd . "./code.cg2js/cg-static-text.fasl")
(#x3a5360a70b0d3ecb902f10b427144ba3 . "./code.cg2js/cg-outline.fasl")
(#x26a51c1846c4538d9b8498347086e671 . "./code.cg2js/cg-gesture-emulation.fasl")
(#x1e4cfb5bba9e4c91f940a59b4b54eb11 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x5d7e6035785a321fee6e870c53f0f827 . "./code.cg2js/cg-picture-widget.fasl")
(#x4c77e51daeeddec6d9ec1e68130fe6cf . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#xa897dad724bd55b1f747009dbbf34472 . "./code.cg2js/cg-chart-widget.fasl")
(#x9c6a04ac8abfd9bf6986220f58e71dbb . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x2a8b59a036e643cbbc3cabd9da7a4051 . "./code.cg2js/cg-lisp-text.fasl")
(#x0186bbc86dc1e68ce385be646f8e2af0 . "./code.cg2js/cg-palette.fasl")
(#xc3670f755794383d2882685375f65fcd . "./code.cg2js/cg-property.fasl")
(#x1df8f14eabfa37bd0cef133919395be2 . "./code.cg2js/cg-intersect-posbox.fasl")
(#xf57d155c550b0a07650a115fea2a5e2e . "./code.cg2js/cg-file-dialog.fasl")
(#x507f0bc74fd8da6fb39fecd14dd967df . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x57ba829215b60a5a07eb7c34eda02483 . "./code.cg2js/cg-get-pixmap.fasl")
(#x7af982afb11403f71fa3769cfd608dee . "./code.cg2js/cg-tray.fasl")
(#x270aedf715f5a75c8071429ad242e917 . "./code.cg2js/cg-acache.fasl")
(#xcaa370698cc58c01624a115626a64e24 . "./code.cg2js/cg-message-dialog.fasl")
(#x613be2f66a2e46648c0fc75405755bd9 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x4e68308878dda46e81310e21b8828670 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x88958af4416dabaa5bfe32007cc690e2 . "./code.cg2js/cg-tab-control.fasl")
(#x24edd2947175d242fd30860c1077eed1 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x48fbac894a845fe45f117e555959c7ba . "./code.cg2js/cg-menu-tooltip.fasl")
(#xa0e98744e07380697bfcd2086d6a7be8 . "./code.cg2js/cg-find-string-dialog.fasl")
(#xa22273bd59a86d0e0139ebaae5ac2411 . "./code.cg2js/cg-menu.fasl")
(#xf1f6740be6e93faed624afe460625734 . "./code.cg2js/cg-get-position.fasl")
(#xfec7042b32d129b645507d6042b65188 . "./code.cg2js/cg-trackbar.fasl")
(#x9e3ac24082584f7514dd93d992c2da21 . "./code.cg2js/cg-selected-object.fasl")
(#x7b324ae459e5231c26f34c0bd7b7f314 . "./code.cg2js/cg-list-view.fasl")
(#x56f34add5f419e93c1d761a16b0e0f16 . "./code.cg2js/cg-html-widget.fasl")
(#x896c2aaee76b48c2e79755395763aa26 . "./code.cg2js/cg-sample-file-menu.fasl")
(#x0fa34312b0a599bb6d36054c303a863a . "./code.cg2js/cg-text-widget.fasl")
(#x0d6c008fb9f8c595b01b86f9ed606640 . "./code.idejs/ide-base.fasl")
(#x4e121c7d68e7617e45bdc129252fa8ef . "./code.idejs/ide-process-browser.fasl")
(#x6e5710e256340591f0d9a86622555f1a . "./code.idejs/ide-patch-dialog.fasl")
(#x44414df5b0ab6059bb184c6d8a12adc1 . "./code.idejs/ide-keyshort.fasl")
(#x46ecf7d46bea1d24666bf194202fc1f5 . "./code.idejs/ide-window-list.fasl")
(#x3b8ce2dc7315ef64875ff4df44707bf6 . "./code.idejs/ide.fasl")
(#x00042ff67c762e62a27bd117ff30090f . "./code.idejs/ide-editor.fasl")
(#xd3c489bc8d8bddabc472a5a876a83a46 . "./code.idejs/ide-clipboard.fasl")
(#xa1720bdd9a8bfc2a4926502b69116d3e . "./code.idejs/ide-trace.fasl")
(#x80f05bf9e2f09f8f5a3bc0f0e2122ae8 . "./code.idejs/ide-class-browser.fasl")
(#xc6ecd62659902b5647a24f0e5c461924 . "./code.idejs/ide-grapher.fasl")
(#xe5f5ef0a2de13f87832a2aea98a86ed5 . "./code.idejs/ide-apropos.fasl")
(#xde48584deb10aa7cd585cb82cf6e9b29 . "./code.idejs/ide-navigator.fasl")
(#xb1252f895b9e15c3cc74453f6a68a22d . "./code.idejs/ide-find-definitions.fasl")
(#x5ca8d166f626ae7ec4d93646a5c3ce3f . "./code.idejs/ide-project.fasl")
(#xd2eb7b5ed977bad51ce09023813f9d7d . "./code.idejs/ide-class-grapher.fasl")
(#x0bafe78b0a71dad2b5d52dba98cf1054 . "./code.idejs/ide-acache.fasl")
(#x40d9884c6d20b41e8f7c0d1281cdbc2f . "./code.idejs/ide-find-in-files.fasl")
(#x82fd56c2a46325ced8f1619e65882bcd . "./code.idejs/ide-inspector.fasl")
(#x2c97006924a0b5e0fd687ce35a1f86d3 . "./code.idejs/ide-intro.fasl")
(#x0410886f75301ff043a9889000678089 . "./code.idejs/ide-package-list.fasl")
(#x1c3c8e21b2d38a2a6dfcd491cfe6574b . "./code.idejs/ide-dev.fasl")
(#x67709feda979fb425c828b577250c386 . "./code.idejs/ide-form.fasl")
(#x9a8f50ffdba20a10156ef75920aa924b . "./cgjs.html")
(#xdac1739373835d1a63cbafa2c3e0b06a . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (format t "Incorrect checksum on file: ~a" (cdr checksum))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows
  (let ((noninteractive (sys:getenv "ACL_NONINTERACTIVE")))
    (when (or (null noninteractive)
	      (string= "" noninteractive))
      (excl::internal-message-box msg "CG/JS Installation" :type :ok))))
